@@ -26,7 +26,7 @@
  * fixed-point AC-3 encoder.
  */
 
-#define CONFIG_FFT_FLOAT 0
+#define FFT_FLOAT 0
 #undef CONFIG_AC3ENC_FLOAT
 #include "internal.h"
 #include "ac3enc.h"
@@ -65,17 +65,6 @@ av_cold int AC3_NAME(mdct_init)(AC3EncodeContext *s)
 }
 
 
-/*
- * Apply KBD window to input samples prior to MDCT.
- */
-static void apply_window(void *dsp, int16_t *output, const int16_t *input,
-                         const int16_t *window, unsigned int len)
-{
-    DSPContext *dsp0 = dsp;
-    dsp0->apply_window_int16(output, input, window, len);
-}
-
-
 /*
  * Normalize the input samples to use the maximum available precision.
  * This assumes signed 16-bit input samples.
@@ -143,6 +132,7 @@ static av_cold int ac3_fixed_encode_init(AVCodecContext *avctx)
 
 AVCodec ff_ac3_fixed_encoder = {
     .name            = "ac3_fixed",
+    .long_name       = NULL_IF_CONFIG_SMALL("ATSC A/52A (AC-3)"),
     .type            = AVMEDIA_TYPE_AUDIO,
     .id              = AV_CODEC_ID_AC3,
     .priv_data_size  = sizeof(AC3EncodeContext),
@@ -151,7 +141,6 @@ AVCodec ff_ac3_fixed_encoder = {
     .close           = ff_ac3_encode_close,
     .sample_fmts     = (const enum AVSampleFormat[]){ AV_SAMPLE_FMT_S16P,
                                                       AV_SAMPLE_FMT_NONE },
-    .long_name       = NULL_IF_CONFIG_SMALL("ATSC A/52A (AC-3)"),
     .priv_class      = &ac3enc_class,
     .channel_layouts = ff_ac3_channel_layouts,
     .defaults        = ac3_defaults,
