@@ -29,7 +29,7 @@
 #include "libavutil/log.h"
 #include "libavutil/time.h"
 #include "fft.h"
-#if CONFIG_FFT_FLOAT
+#if FFT_FLOAT
 #include "dct.h"
 #include "rdft.h"
 #endif
@@ -37,6 +37,7 @@
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -50,7 +51,7 @@
    pim += (MUL16(are, bim) + MUL16(bre, aim));\
 }
 
-#if CONFIG_FFT_FLOAT
+#if FFT_FLOAT
 #   define RANGE 1.0
 #   define REF_SCALE(x, bits)  (x)
 #   define FMT "%10.6f"
@@ -112,7 +113,6 @@ static void fft_ref(FFTComplex *tabr, FFTComplex *tab, int nbits)
     }
 }
 
-#if CONFIG_MDCT
 static void imdct_ref(FFTSample *out, FFTSample *in, int nbits)
 {
     int n = 1<<nbits;
@@ -147,10 +147,8 @@ static void mdct_ref(FFTSample *output, FFTSample *input, int nbits)
         output[k] = REF_SCALE(s, nbits - 1);
     }
 }
-#endif /* CONFIG_MDCT */
 
-#if CONFIG_FFT_FLOAT
-#if CONFIG_DCT
+#if FFT_FLOAT
 static void idct_ref(float *output, float *input, int nbits)
 {
     int n = 1<<nbits;
@@ -183,7 +181,6 @@ static void dct_ref(float *output, float *input, int nbits)
         output[k] = s;
     }
 }
-#endif /* CONFIG_DCT */
 #endif
 
 
@@ -251,7 +248,7 @@ int main(int argc, char **argv)
     int do_inverse = 0;
     FFTContext s1, *s = &s1;
     FFTContext m1, *m = &m1;
-#if CONFIG_FFT_FLOAT
+#if FFT_FLOAT
     RDFTContext r1, *r = &r1;
     DCTContext d1, *d = &d1;
     int fft_size_2;
@@ -307,7 +304,6 @@ int main(int argc, char **argv)
     tab2 = av_malloc(fft_size * sizeof(FFTSample));
 
     switch (transform) {
-#if CONFIG_MDCT
     case TRANSFORM_MDCT:
         av_log(NULL, AV_LOG_INFO,"Scale factor is set to %f\n", scale);
         if (do_inverse)
@@ -316,7 +312,6 @@ int main(int argc, char **argv)
             av_log(NULL, AV_LOG_INFO,"MDCT");
         ff_mdct_init(m, fft_nbits, do_inverse, scale);
         break;
-#endif /* CONFIG_MDCT */
     case TRANSFORM_FFT:
         if (do_inverse)
             av_log(NULL, AV_LOG_INFO,"IFFT");
@@ -325,8 +320,7 @@ int main(int argc, char **argv)
         ff_fft_init(s, fft_nbits, do_inverse);
         fft_ref_init(fft_nbits, do_inverse);
         break;
-#if CONFIG_FFT_FLOAT
-#if CONFIG_RDFT
+#if FFT_FLOAT
     case TRANSFORM_RDFT:
         if (do_inverse)
             av_log(NULL, AV_LOG_INFO,"IDFT_C2R");
@@ -335,8 +329,6 @@ int main(int argc, char **argv)
         ff_rdft_init(r, fft_nbits, do_inverse ? IDFT_C2R : DFT_R2C);
         fft_ref_init(fft_nbits, do_inverse);
         break;
-#endif /* CONFIG_RDFT */
-#if CONFIG_DCT
     case TRANSFORM_DCT:
         if (do_inverse)
             av_log(NULL, AV_LOG_INFO,"DCT_III");
@@ -344,7 +336,6 @@ int main(int argc, char **argv)
             av_log(NULL, AV_LOG_INFO,"DCT_II");
         ff_dct_init(d, fft_nbits, do_inverse ? DCT_III : DCT_II);
         break;
-#endif /* CONFIG_DCT */
 #endif
     default:
         av_log(NULL, AV_LOG_ERROR, "Requested transform not supported\n");
@@ -363,7 +354,6 @@ int main(int argc, char **argv)
     av_log(NULL, AV_LOG_INFO,"Checking...\n");
 
     switch (transform) {
-#if CONFIG_MDCT
     case TRANSFORM_MDCT:
         if (do_inverse) {
             imdct_ref((FFTSample *)tab_ref, (FFTSample *)tab1, fft_nbits);
@@ -377,7 +367,6 @@ int main(int argc, char **argv)
             err = check_diff((FFTSample *)tab_ref, tab2, fft_size / 2, scale);
         }
         break;
-#endif /* CONFIG_MDCT */
     case TRANSFORM_FFT:
         memcpy(tab, tab1, fft_size * sizeof(FFTComplex));
         s->fft_permute(s, tab);
@@ -386,8 +375,7 @@ int main(int argc, char **argv)
         fft_ref(tab_ref, tab1, fft_nbits);
         err = check_diff((FFTSample *)tab_ref, (FFTSample *)tab, fft_size * 2, 1.0);
         break;
-#if CONFIG_FFT_FLOAT
-#if CONFIG_RDFT
+#if FFT_FLOAT
     case TRANSFORM_RDFT:
         fft_size_2 = fft_size >> 1;
         if (do_inverse) {
@@ -419,8 +407,6 @@ int main(int argc, char **argv)
             err = check_diff((float *)tab_ref, (float *)tab2, fft_size, 1.0);
         }
         break;
-#endif /* CONFIG_RDFT */
-#if CONFIG_DCT
     case TRANSFORM_DCT:
         memcpy(tab, tab1, fft_size * sizeof(FFTComplex));
         d->dct_calc(d, tab);
@@ -431,7 +417,6 @@ int main(int argc, char **argv)
         }
         err = check_diff((float *)tab_ref, (float *)tab, fft_size, 1.0);
         break;
-#endif /* CONFIG_DCT */
 #endif
     }
 
@@ -459,7 +444,7 @@ int main(int argc, char **argv)
                     memcpy(tab, tab1, fft_size * sizeof(FFTComplex));
                     s->fft_calc(s, tab);
                     break;
-#if CONFIG_FFT_FLOAT
+#if FFT_FLOAT
                 case TRANSFORM_RDFT:
                     memcpy(tab2, tab1, fft_size * sizeof(FFTSample));
                     r->rdft_calc(r, tab2);
@@ -483,25 +468,19 @@ int main(int argc, char **argv)
     }
 
     switch (transform) {
-#if CONFIG_MDCT
     case TRANSFORM_MDCT:
         ff_mdct_end(m);
         break;
-#endif /* CONFIG_MDCT */
     case TRANSFORM_FFT:
         ff_fft_end(s);
         break;
-#if CONFIG_FFT_FLOAT
-#if CONFIG_RDFT
+#if FFT_FLOAT
     case TRANSFORM_RDFT:
         ff_rdft_end(r);
         break;
-#endif /* CONFIG_RDFT */
-#if CONFIG_DCT
     case TRANSFORM_DCT:
         ff_dct_end(d);
         break;
-#endif /* CONFIG_DCT */
 #endif
     }
 
@@ -511,5 +490,8 @@ int main(int argc, char **argv)
     av_free(tab_ref);
     av_free(exptab);
 
-    return err;
+    if (err)
+        printf("Error: %d.\n", err);
+
+    return !!err;
 }
