@@ -23,7 +23,7 @@
 #ifndef AVCODEC_PRORESDSP_H
 #define AVCODEC_PRORESDSP_H
 
-#include "dsputil.h"
+#include <stdint.h>
 
 #define PRORES_BITS_PER_SAMPLE 10 ///< output precision of prores decoder
 
@@ -32,8 +32,8 @@ typedef struct ProresDSPContext {
     uint8_t idct_permutation[64];
     int dct_permutation_type;
     uint8_t dct_permutation[64];
-    void (* idct_put) (uint16_t *out, int linesize, DCTELEM *block, const int16_t *qmat);
-    void (* fdct) (const uint16_t *src, int linesize, DCTELEM *block);
+    void (* idct_put) (uint16_t *out, int linesize, int16_t *block, const int16_t *qmat);
+    void (* fdct) (const uint16_t *src, int linesize, int16_t *block);
 } ProresDSPContext;
 
 void ff_proresdsp_init(ProresDSPContext *dsp);
