@@ -130,7 +130,7 @@ static int img_read_probe(AVProbeData *p)
         if (av_filename_number_test(p->filename))
             return AVPROBE_SCORE_MAX;
         else
-            return AVPROBE_SCORE_MAX / 2;
+            return AVPROBE_SCORE_EXTENSION;
     }
     return 0;
 }
@@ -194,7 +194,7 @@ static int img_read_header(AVFormatContext *s1)
             return AVERROR(ENOENT);
         s->img_first  = first_index;
         s->img_last   = last_index;
-        s->img_number = s->start_number != 1 ? s->start_number : first_index;
+        s->img_number = first_index;
         /* compute duration */
         st->start_time = 0;
         st->duration   = last_index - first_index + 1;
